import _Button from "antd/es/button";
import _extends from "@babel/runtime/helpers/extends";
import { RecursionField, useField } from '@formily/react';
import { isArr, isUndef } from '@formily/shared';
import cls from 'classnames';
import React, { createContext, forwardRef, Fragment, memo, useContext, useMemo } from 'react';
import useStyle from './style';
import { usePrefixCls } from '@formily/antd-v5/lib/__builtins__';
import { ArrayBase } from '@formily/antd-v5';
import { useEditor, useSelected } from '../../../../hooks';
import { EditOutlined, PlusOutlined } from '@ant-design/icons';
import { isString, omit, set } from 'lodash';
export var isIndexComponent = function isIndexComponent(schema) {
  var _schema$xComponent, _schema$xComponent$in;
  return ((_schema$xComponent = schema['x-component']) === null || _schema$xComponent === void 0 ? void 0 : (_schema$xComponent$in = _schema$xComponent.indexOf) === null || _schema$xComponent$in === void 0 ? void 0 : _schema$xComponent$in.call(_schema$xComponent, 'Index')) > -1;
};
export var isColumnComponent = function isColumnComponent(schema) {
  var _schema$xComponent2;
  return ((_schema$xComponent2 = schema['x-component']) === null || _schema$xComponent2 === void 0 ? void 0 : _schema$xComponent2.indexOf('Column')) > -1;
};
export var isSidePanelComponent = function isSidePanelComponent(schema) {
  var _schema$xComponent3;
  return ((_schema$xComponent3 = schema['x-component']) === null || _schema$xComponent3 === void 0 ? void 0 : _schema$xComponent3.indexOf('SidePanel')) > -1;
};
export var isSideAdditionComponent = function isSideAdditionComponent(schema) {
  var _schema$xComponent4;
  return ((_schema$xComponent4 = schema['x-component']) === null || _schema$xComponent4 === void 0 ? void 0 : _schema$xComponent4.indexOf('AddSidePanel')) > -1;
};
export var isOperationsComponent = function isOperationsComponent(schema) {
  var _schema$xComponent5;
  return ((_schema$xComponent5 = schema['x-component']) === null || _schema$xComponent5 === void 0 ? void 0 : _schema$xComponent5.indexOf('Operations')) > -1;
};
export var isAdditionComponent = function isAdditionComponent(schema) {
  var _schema$xComponent6;
  return ((_schema$xComponent6 = schema['x-component']) === null || _schema$xComponent6 === void 0 ? void 0 : _schema$xComponent6.indexOf('Addition')) > -1;
};
var isRemoveComponent = function isRemoveComponent(schema) {
  var _schema$xComponent7, _schema$xComponent7$i;
  return ((_schema$xComponent7 = schema['x-component']) === null || _schema$xComponent7 === void 0 ? void 0 : (_schema$xComponent7$i = _schema$xComponent7.indexOf) === null || _schema$xComponent7$i === void 0 ? void 0 : _schema$xComponent7$i.call(_schema$xComponent7, 'Remove')) > -1;
};
var isMoveUpComponent = function isMoveUpComponent(schema) {
  var _schema$xComponent8, _schema$xComponent8$i;
  return ((_schema$xComponent8 = schema['x-component']) === null || _schema$xComponent8 === void 0 ? void 0 : (_schema$xComponent8$i = _schema$xComponent8.indexOf) === null || _schema$xComponent8$i === void 0 ? void 0 : _schema$xComponent8$i.call(_schema$xComponent8, 'MoveUp')) > -1;
};
var isMoveDownComponent = function isMoveDownComponent(schema) {
  var _schema$xComponent9, _schema$xComponent9$i;
  return ((_schema$xComponent9 = schema['x-component']) === null || _schema$xComponent9 === void 0 ? void 0 : (_schema$xComponent9$i = _schema$xComponent9.indexOf) === null || _schema$xComponent9$i === void 0 ? void 0 : _schema$xComponent9$i.call(_schema$xComponent9, 'MoveDown')) > -1;
};
export var isOperationComponent = function isOperationComponent(schema) {
  return isAdditionComponent(schema) || isRemoveComponent(schema) || isMoveDownComponent(schema) || isMoveUpComponent(schema);
};
var getDisplayStatus = function getDisplayStatus(mode, columnEnv) {
  var isMatch = isString(columnEnv) ? columnEnv === mode : columnEnv.includes(mode);
  return mode === 'normal' ? isMatch ? 'visible' : 'hidden' : isMatch ? 'visible' : 'none';
};
export var useArrayTableSources = function useArrayTableSources(arrayField, schema, mode) {
  if (mode === void 0) {
    mode = 'normal';
  }
  return useMemo(function () {
    var _parseSources = function parseSources(schema) {
      if (isColumnComponent(schema) || isOperationsComponent(schema) || isAdditionComponent(schema)) {
        var _schema$xComponentP, _field$component;
        var xComponentProps = schema['x-component-props'] || {};
        if (!xComponentProps['dataIndex'] && !schema['name']) return [];
        var _display = undefined;
        var name = ((_schema$xComponentP = schema['x-component-props']) === null || _schema$xComponentP === void 0 ? void 0 : _schema$xComponentP['dataIndex']) || schema['name'];
        var field = arrayField.query(arrayField.address.concat(name)).take();
        if (mode === 'normal') {
          var columnEnv = xComponentProps['env'];
          _display = columnEnv ? getDisplayStatus(mode, columnEnv) : 'none';
        } else {
          var _columnEnv = xComponentProps['env'];
          if (_columnEnv) {
            _display = _columnEnv ? getDisplayStatus(mode, _columnEnv) : 'none';
          }
        }
        var columnProps = (field === null || field === void 0 ? void 0 : (_field$component = field.component) === null || _field$component === void 0 ? void 0 : _field$component[1]) || xComponentProps;
        var display = _display || (field === null || field === void 0 ? void 0 : field.display) || schema['x-display'];
        return [{
          name: name,
          display: display,
          field: field,
          schema: schema,
          columnProps: columnProps
        }];
      } else if (schema.properties) {
        return schema.reduceProperties(function (buf, schema) {
          return buf.concat(_parseSources(schema));
        }, []);
      }
      return [];
    };
    var parseArrayItems = function parseArrayItems(schema) {
      if (!schema) return [];
      var sources = [];
      var items = isArr(schema) ? schema : [schema];
      return items.reduce(function (columns, schema) {
        var item = _parseSources(schema);
        if (item) {
          return columns.concat(item);
        }
        return columns;
      }, sources);
    };
    if (!schema) throw new Error('can not found schema object');
    return parseArrayItems(schema.items);
  }, [arrayField, schema, mode]);
};
var createCellEditor = function createCellEditor(field, schema) {
  var CellEditor = /*#__PURE__*/memo(function (props) {
    var _field$query;
    var index = props.data[indexKey];
    var keys = [];
    schema.mapProperties(function (property, key) {
      if (property.type !== 'boolean') {
        keys.push(key);
      }
    });
    var path = field.address.concat(index + '.' + keys[0]);
    var self = (_field$query = field.query(path)) === null || _field$query === void 0 ? void 0 : _field$query.take();
    self === null || self === void 0 ? void 0 : self.setPattern('editable');
    var children = /*#__PURE__*/React.createElement(ArrayBase.Item, {
      index: index,
      record: function record() {
        var _field$value;
        return field === null || field === void 0 ? void 0 : (_field$value = field.value) === null || _field$value === void 0 ? void 0 : _field$value[index];
      }
    }, /*#__PURE__*/React.createElement(RecursionField, {
      schema: schema,
      name: index,
      onlyRenderProperties: true
    }));
    return index > -1 ? children : null;
  });
  return CellEditor;
};
export var LOWCODE_NODE_ID = '__lowcode_node_id__';
export var indexKey = '__DO_NOT_USE_THIS_PROPERTY_index__';
export var useArrayTableColumns = function useArrayTableColumns(dataSource, field, sources) {
  return useMemo(function () {
    return sources.reduce(function (buf, _ref, key) {
      var name = _ref.name,
        columnProps = _ref.columnProps,
        schema = _ref.schema,
        display = _ref.display;
      if (display !== 'visible') return buf;
      if (!isColumnComponent(schema)) return buf;
      var render = undefined;
      var fieldName = undefined;
      var editable = columnProps.editable;
      var editableUi = false;
      // 不可编辑时，用 render 来渲染单元格
      if (!editable) {
        render = function render(value, record) {
          var index = record[indexKey];
          var children = /*#__PURE__*/React.createElement(ArrayBase.Item, {
            index: index,
            record: function record() {
              var _field$value2;
              return field === null || field === void 0 ? void 0 : (_field$value2 = field.value) === null || _field$value2 === void 0 ? void 0 : _field$value2[index];
            }
          }, /*#__PURE__*/React.createElement(RecursionField, {
            schema: schema,
            name: index,
            onlyRenderProperties: true
          }));
          return index > -1 ? children : null;
        };
      } else {
        // field?.setPattern('readPretty');
        // 可编辑时，用 cellEditor 来渲染单元格
        fieldName = name;
        var keys = [];
        schema.mapProperties(function (property, key) {
          if (property.type !== 'boolean') {
            property['x-pattern'] = 'readPretty';
            keys.push(key);
          } else {
            editable = false;
            editableUi = true;
          }
        });
        render = function render(value, record) {
          var _field$query2;
          var index = record[indexKey];
          var path = field.address.concat(index + '.' + keys[0]);
          var self = (_field$query2 = field.query(path)) === null || _field$query2 === void 0 ? void 0 : _field$query2.take();
          self === null || self === void 0 ? void 0 : self.setPattern('readPretty');
          var children = /*#__PURE__*/React.createElement(ArrayBase.Item, {
            index: index,
            record: function record() {
              var _field$value3;
              return field === null || field === void 0 ? void 0 : (_field$value3 = field.value) === null || _field$value3 === void 0 ? void 0 : _field$value3[index];
            }
          }, /*#__PURE__*/React.createElement(RecursionField, {
            schema: schema,
            name: index,
            onlyRenderProperties: true
          }));
          return index > -1 ? children : null;
        };
      }
      return buf.concat(_extends({}, omit(columnProps, 'env'), {
        editable: editable,
        fieldName: fieldName,
        cellEditor: createCellEditor(field, schema),
        cellClassRules: {
          'gant-grid-cell-edit': function gantGridCellEdit(params) {
            return editable || editableUi;
          }
        },
        render: render
      }));
    }, []);
  }, [field, sources]);
};
export var useAddition = function useAddition(schema) {
  return schema.reduceProperties(function (addition, schema, key) {
    if (isAdditionComponent(schema)) {
      return /*#__PURE__*/React.createElement(RecursionField, {
        schema: schema,
        name: key
      });
    }
    return addition;
  }, null);
};
export var FieldDataSourceContext = /*#__PURE__*/createContext(undefined);
export var useFieldDataSource = function useFieldDataSource() {
  return useContext(FieldDataSourceContext);
};
export var ValueModeContext = /*#__PURE__*/createContext('');
export var useValueMode = function useValueMode() {
  return useContext(ValueModeContext);
};
export var RowSelectedContext = /*#__PURE__*/createContext([]);
export var useRowSelectedContext = function useRowSelectedContext() {
  return useContext(RowSelectedContext);
};
export var GridManagerContext = /*#__PURE__*/createContext({
  current: null
});
export var useGridManagerRef = function useGridManagerRef() {
  return useContext(GridManagerContext);
};
export var isOpenComponent = function isOpenComponent(schema) {
  var _schema$xComponent10;
  return ((_schema$xComponent10 = schema['x-component']) === null || _schema$xComponent10 === void 0 ? void 0 : _schema$xComponent10.indexOf('Open')) > -1;
};
export var Navigation = /*#__PURE__*/memo(/*#__PURE__*/forwardRef(function (props, ref) {
  var _array$field;
  var record = ArrayBase.useRecord();
  var self = useField();
  var array = ArrayBase.useArray();
  var prefixCls = usePrefixCls('formily-array-advance-table');
  var _useStyle = useStyle(prefixCls),
    wrapSSR = _useStyle[0],
    hashId = _useStyle[1];
  var editor = useEditor();
  if (!array) return null;
  if (((_array$field = array.field) === null || _array$field === void 0 ? void 0 : _array$field.pattern) !== 'editable') return null;
  return wrapSSR(/*#__PURE__*/React.createElement(_Button, _extends({
    type: "ghost"
  }, props, {
    style: _extends({
      width: 'auto',
      height: 'auto'
    }, props.style),
    disabled: self === null || self === void 0 ? void 0 : self.disabled,
    className: cls(prefixCls + "-navigation", hashId, self !== null && self !== void 0 && self.disabled ? prefixCls + "-navigation-disabled" : '', props.className),
    ref: ref,
    onClick: function onClick(e) {
      var _editor$get, _editor$get$currentDo, _editor$get$currentDo2;
      if (self !== null && self !== void 0 && self.disabled || !record) return;
      var nodeId = record[LOWCODE_NODE_ID];
      (_editor$get = editor.get('project')) === null || _editor$get === void 0 ? void 0 : (_editor$get$currentDo = _editor$get.currentDocument) === null || _editor$get$currentDo === void 0 ? void 0 : (_editor$get$currentDo2 = _editor$get$currentDo.getNodeById(nodeId)) === null || _editor$get$currentDo2 === void 0 ? void 0 : _editor$get$currentDo2.select();
      e.stopPropagation();
      if (props.onClick) {
        props.onClick(e);
        if (e.defaultPrevented) return;
      }
    },
    icon: isUndef(props.icon) ? /*#__PURE__*/React.createElement(EditOutlined, null) : props.icon
  }), props.title || self.title));
}));
export var Column = function Column() {
  return /*#__PURE__*/React.createElement(Fragment, null);
};
export var SidePanel = function SidePanel(props) {
  return props.children;
};
export var Addition = /*#__PURE__*/memo(function (props) {
  var _array$field2, _array$field3;
  var selected = useSelected();
  var self = useField();
  var array = ArrayBase.useArray();
  var valueMode = useValueMode();
  var prefixCls = usePrefixCls('formily-array-advance-table');
  var _useStyle2 = useStyle(prefixCls),
    wrapSSR = _useStyle2[0],
    hashId = _useStyle2[1];
  if (!array) return null;
  if (((_array$field2 = array.field) === null || _array$field2 === void 0 ? void 0 : _array$field2.pattern) !== 'editable' && ((_array$field3 = array.field) === null || _array$field3 === void 0 ? void 0 : _array$field3.pattern) !== 'disabled') return null;
  var value = array.field.value;
  var fieldNames = array.field.componentProps.fieldNames || {
    label: 'label',
    value: 'value'
  };
  return wrapSSR(/*#__PURE__*/React.createElement(_Button, _extends({
    color: "primary",
    variant: "filled",
    size: "small"
  }, props, {
    disabled: self === null || self === void 0 ? void 0 : self.disabled,
    className: cls(prefixCls + "-addition", hashId, props.className),
    onClick: function onClick(e) {
      var _array$props, _extends2;
      if ((_array$props = array.props) !== null && _array$props !== void 0 && _array$props.disabled) return;
      var defaultValue = _extends((_extends2 = {}, _extends2[indexKey] = value.length, _extends2), props.defaultValue);
      if (valueMode === 'children') {
        var _selected$document;
        var addNode = (_selected$document = selected.document) === null || _selected$document === void 0 ? void 0 : _selected$document.insertNode(selected, {
          componentName: 'Input',
          props: defaultValue
        });
        defaultValue[LOWCODE_NODE_ID] = addNode.id;
        console.log('--defaultValue', props, defaultValue);
        addNode.setPropValue(fieldNames.label, addNode.title + '');
        addNode.setPropValue(fieldNames.value, addNode.id);
        set(defaultValue, fieldNames.label, addNode.title);
        set(defaultValue, fieldNames.value, addNode.id);
      }
      if (props.method === 'unshift') {
        var _array$field4, _array$field4$unshift, _array$props2, _array$props2$onAdd;
        (_array$field4 = array.field) === null || _array$field4 === void 0 ? void 0 : (_array$field4$unshift = _array$field4.unshift) === null || _array$field4$unshift === void 0 ? void 0 : _array$field4$unshift.call(_array$field4, defaultValue);
        (_array$props2 = array.props) === null || _array$props2 === void 0 ? void 0 : (_array$props2$onAdd = _array$props2.onAdd) === null || _array$props2$onAdd === void 0 ? void 0 : _array$props2$onAdd.call(_array$props2, 0);
      } else {
        var _array$field5, _array$field5$push, _array$props3, _array$props3$onAdd, _array$field6, _array$field6$value;
        (_array$field5 = array.field) === null || _array$field5 === void 0 ? void 0 : (_array$field5$push = _array$field5.push) === null || _array$field5$push === void 0 ? void 0 : _array$field5$push.call(_array$field5, defaultValue);
        (_array$props3 = array.props) === null || _array$props3 === void 0 ? void 0 : (_array$props3$onAdd = _array$props3.onAdd) === null || _array$props3$onAdd === void 0 ? void 0 : _array$props3$onAdd.call(_array$props3, (array === null || array === void 0 ? void 0 : (_array$field6 = array.field) === null || _array$field6 === void 0 ? void 0 : (_array$field6$value = _array$field6.value) === null || _array$field6$value === void 0 ? void 0 : _array$field6$value.length) - 1);
      }
      if (props.onClick) {
        props.onClick(e);
      }
    },
    icon: isUndef(props.icon) ? /*#__PURE__*/React.createElement(PlusOutlined, null) : props.icon
  }), props.title || self.title));
});
export var BatchRemove = /*#__PURE__*/memo(function (props) {
  var _array$field7, _array$field8;
  var rowSelected = useRowSelectedContext();
  var selected = useSelected();
  var self = useField();
  var array = ArrayBase.useArray();
  var valueMode = useValueMode();
  var prefixCls = usePrefixCls('formily-array-advance-table');
  var _useStyle3 = useStyle(prefixCls),
    wrapSSR = _useStyle3[0],
    hashId = _useStyle3[1];
  if (!array) return null;
  if (((_array$field7 = array.field) === null || _array$field7 === void 0 ? void 0 : _array$field7.pattern) !== 'editable' && ((_array$field8 = array.field) === null || _array$field8 === void 0 ? void 0 : _array$field8.pattern) !== 'disabled') return null;
  return wrapSSR(/*#__PURE__*/React.createElement(_Button, _extends({
    color: "danger",
    variant: "filled",
    size: "small"
  }, props, {
    disabled: (self === null || self === void 0 ? void 0 : self.disabled) || rowSelected.length === 0,
    className: cls(prefixCls + "-remove", hashId, self !== null && self !== void 0 && self.disabled ? prefixCls + "-remove-disabled" : '', props.className),
    onClick: function onClick(e) {
      if (self !== null && self !== void 0 && self.disabled) return;
      e.stopPropagation();
      if (props.onClick) {
        props.onClick(e);
        if (e.defaultPrevented) return;
      }
      var value = array.field.value;
      rowSelected.forEach(function (key) {
        var index = value.findIndex(function (item) {
          return item[LOWCODE_NODE_ID] === key;
        });
        if (index > -1) {
          var _array$field9, _array$field9$remove, _array$props4, _array$props4$onRemov;
          if (valueMode === 'children') {
            var _selected$children$ge;
            (_selected$children$ge = selected.children.get(index)) === null || _selected$children$ge === void 0 ? void 0 : _selected$children$ge.remove();
          }
          (_array$field9 = array.field) === null || _array$field9 === void 0 ? void 0 : (_array$field9$remove = _array$field9.remove) === null || _array$field9$remove === void 0 ? void 0 : _array$field9$remove.call(_array$field9, index);
          (_array$props4 = array.props) === null || _array$props4 === void 0 ? void 0 : (_array$props4$onRemov = _array$props4.onRemove) === null || _array$props4$onRemov === void 0 ? void 0 : _array$props4$onRemov.call(_array$props4, index);
        }
      });
    },
    icon: props.icon
  }), props.title || self.title));
});
export var isActionComponent = function isActionComponent(schema) {
  var _schema$xComponent11;
  return ((_schema$xComponent11 = schema['x-component']) === null || _schema$xComponent11 === void 0 ? void 0 : _schema$xComponent11.indexOf('Action')) > -1;
};
export var isBatchRemoveComponent = function isBatchRemoveComponent(schema) {
  var _schema$xComponent12;
  return ((_schema$xComponent12 = schema['x-component']) === null || _schema$xComponent12 === void 0 ? void 0 : _schema$xComponent12.indexOf('BatchRemove')) > -1;
};
export var Action = function Action(_ref2) {
  var children = _ref2.children;
  return children;
};
export var useAction = function useAction(schema) {
  return schema.reduceProperties(function (addition, schema, key) {
    if (isActionComponent(schema)) {
      return /*#__PURE__*/React.createElement(RecursionField, {
        schema: schema,
        name: key
      });
    }
    return addition;
  }, null);
};
export var useBatchRemove = function useBatchRemove(schema) {
  return schema.reduceProperties(function (batchRemove, schema, key) {
    if (isBatchRemoveComponent(schema)) {
      return /*#__PURE__*/React.createElement(RecursionField, {
        schema: schema,
        name: key
      });
    }
    return batchRemove;
  }, null);
};