import _Dropdown from "antd/es/dropdown";
import _Button from "antd/es/button";
import _extends from "@babel/runtime/helpers/extends";
import { ArrayBase, ArrayItems } from '@formily/antd-v5';
import { usePrefixCls } from '@formily/antd-v5/lib/__builtins__';
import { Schema } from '@formily/json-schema';
import { observer, SchemaContext, useField, useFieldSchema } from '@formily/react';
import React, { forwardRef, useContext, useMemo } from 'react';
import cls from 'classnames';
import { useStyle } from './style';
import { FormPath, isUndef } from '@formily/shared';
import { isEmpty, set } from 'lodash';
import { DeleteOutlined } from '@ant-design/icons';
import { SideBaseContext, useIndex } from '../array-advance-table/validator-setter';
import { useSelected } from '../../../../hooks';

// 内置类型与自定义格式映射
var customFormatMap = {
  // URL 地址
  url: /^(https?:\/\/)?([\w\-]+\.)+[\w\-]+(\/[\w\-\.\/?%&=]*)?$/,
  // 邮箱格式
  email: /^[\w-+.]+@[\w-]+\.[\w-.]+$/,
  // 身份证（简单校验，15 或 18 位，末尾可X）
  idcard: /^(?:\d{15}|\d{17}[0-9X])$/,
  // 手机号（中国大陆）
  phone: /^1[3-9]\d{9}$/,
  // 货币金额（最多两位小数）
  money: /^\d+(?:\.\d{1,2})?$/,
  // 中文字符
  zh: /^[\u4e00-\u9fa5]+$/,
  // 日期（YYYY-MM-DD）
  date: /^\d{4}-\d{2}-\d{2}$/,
  // 邮编（中国大陆 6 位）
  zip: /^\d{6}$/,
  // 数字（整数或小数）
  number: /^-?\d+(?:\.\d+)?$/,
  // 整数
  integer: /^-?\d+$/,
  // 浮点数（必须含小数点）
  "float": /^-?\d+\.\d+$/
};
var ruleFormats = [{
  label: 'URL地址',
  value: 'url'
}, {
  label: '邮箱',
  value: 'email'
}, {
  label: '数字',
  value: 'number'
}, {
  label: '整数',
  value: 'integer'
}, {
  label: '小数',
  value: 'float'
}, {
  label: '身份证',
  value: 'idcard'
}, {
  label: '手机号',
  value: 'phone'
}, {
  label: '货币',
  value: 'money'
}, {
  label: '中文',
  value: 'zh'
}, {
  label: '日期',
  value: 'date'
}, {
  label: '邮编',
  value: 'zip'
}];
var ruleSchema = {
  type: 'array',
  items: {
    type: 'object',
    'x-decorator': 'RuleGroup',
    'x-component': 'FormLayout',
    'x-component-props': {
      feedbackLayout: 'terse',
      labelCol: 6,
      colon: false,
      labelAlign: 'left'
    },
    properties: {
      layout: {
        type: 'void',
        'x-component': 'FormRow',
        properties: {
          col1: {
            type: 'void',
            'x-component': 'FormRow.Col',
            'x-component-props': {
              flex: '25px',
              style: {
                display: 'flex',
                alignItems: 'center'
              }
            },
            properties: {
              sort: {
                type: 'void',
                'x-decorator': 'FormItem',
                'x-component': 'ArrayItems.SortHandle'
              }
            }
          },
          col2: {
            type: 'void',
            'x-component': 'FormRow.Col',
            'x-component-props': {
              flex: 'auto'
            },
            properties: {
              __ruleType__: {
                type: 'string',
                'x-hidden': true,
                'x-component': 'Input'
              },
              requiredGroup: {
                type: 'void',
                'x-hidden': '{{ !($self.query(".__ruleType__").value() == "required") }}',
                properties: {
                  required: {
                    type: 'string',
                    title: '必填',
                    'x-decorator': 'FormItem',
                    'x-component': 'Switch',
                    'x-hidden': true
                  },
                  message: {
                    type: 'string',
                    title: '必填',
                    'x-decorator': 'FormItem',
                    'x-component': 'Input',
                    'x-component-props': {
                      allowClear: true
                    }
                  }
                }
              },
              patternGroup: {
                type: 'void',
                'x-hidden': '{{ !($self.query(".__ruleType__").value() === "pattern") }}',
                properties: {
                  'pattern.type': {
                    type: 'string',
                    'x-hidden': true,
                    "default": 'JSExpression'
                  },
                  'pattern.value': {
                    type: 'string',
                    title: '正则',
                    'x-decorator': 'FormItem',
                    'x-component': 'Input'
                    // 'x-component-props': {
                    //   prefix: '/',
                    //   suffix: '/',
                    // },
                  },
                  message: {
                    type: 'string',
                    title: '提示信息',
                    'x-decorator': 'FormItem',
                    'x-component': 'Input',
                    'x-component-props': {
                      allowClear: true,
                      placeholder: '提示信息'
                    }
                  }
                }
              },
              formatGroup: {
                type: 'void',
                'x-hidden': '{{ !($self.query(".__ruleType__").value() === "__format__") }}',
                properties: {
                  __format__: {
                    type: 'string',
                    title: '格式',
                    "enum": ruleFormats,
                    'x-decorator': 'FormItem',
                    'x-component': 'Select',
                    'x-component-props': {
                      allowClear: true,
                      placeholder: '格式'
                    }
                  },
                  pattern: {
                    type: 'string',
                    'x-hidden': true,
                    'x-data': customFormatMap,
                    'x-reactions': [{
                      dependencies: ['.__format__'],
                      fulfill: {
                        state: {
                          value: "{{ $self.data[$deps[0]] }}"
                        }
                      }
                    }]
                  },
                  message: {
                    type: 'string',
                    title: '提示信息',
                    'x-decorator': 'FormItem',
                    'x-component': 'Input',
                    'x-component-props': {
                      allowClear: true,
                      placeholder: '提示信息'
                    },
                    'x-data': ruleFormats.reduce(function (acc, cur) {
                      var _extends2;
                      return _extends({}, acc, (_extends2 = {}, _extends2[cur.value] = cur.label, _extends2));
                    }, {}),
                    'x-reactions': [{
                      dependencies: ['.__format__'],
                      fulfill: {
                        state: {
                          value: "{{ $deps[0] ? \"\u4E0D\u7B26\u5408\" + $self.data[$deps[0]] + \"\u683C\u5F0F\" : \"\" }}"
                        }
                      }
                    }]
                  }
                }
              },
              lenGroup: {
                type: 'void',
                'x-hidden': '{{ !($self.query(".__ruleType__").value() === "len") }}',
                properties: {
                  len: {
                    type: 'string',
                    title: '长度',
                    'x-decorator': 'FormItem',
                    'x-component': 'InputNumber'
                  },
                  message: {
                    type: 'string',
                    title: '提示信息',
                    'x-decorator': 'FormItem',
                    'x-component': 'Input',
                    'x-component-props': {
                      allowClear: true,
                      placeholder: '提示信息'
                    },
                    'x-reactions': [{
                      dependencies: ['.len'],
                      fulfill: {
                        state: {
                          value: "{{ $deps[0] ? \"\u957F\u5EA6\u53EA\u80FD\u4E3A\" + $deps[0] : \"\" }}"
                        }
                      }
                    }]
                  }
                }
              },
              minGroup: {
                type: 'void',
                'x-hidden': '{{ !($self.query(".__ruleType__").value() === "min") }}',
                properties: {
                  min: {
                    type: 'string',
                    title: '最小值',
                    'x-decorator': 'FormItem',
                    'x-component': 'InputNumber'
                  },
                  message: {
                    type: 'string',
                    title: '提示信息',
                    'x-decorator': 'FormItem',
                    'x-component': 'Input',
                    'x-component-props': {
                      allowClear: true,
                      placeholder: '提示信息'
                    },
                    'x-reactions': [{
                      dependencies: ['.min'],
                      fulfill: {
                        state: {
                          value: "{{ $deps[0] ? \"\u6700\u5C0F\u503C\u4E3A\" + $deps[0] : \"\" }}"
                        }
                      }
                    }]
                  }
                }
              },
              maxGroup: {
                type: 'void',
                'x-hidden': '{{ !($self.query(".__ruleType__").value() === "max") }}',
                properties: {
                  max: {
                    type: 'string',
                    title: '最大值',
                    'x-decorator': 'FormItem',
                    'x-component': 'InputNumber'
                  },
                  message: {
                    type: 'string',
                    title: '提示信息',
                    'x-decorator': 'FormItem',
                    'x-component': 'Input',
                    'x-component-props': {
                      allowClear: true,
                      placeholder: '提示信息'
                    },
                    'x-reactions': [{
                      dependencies: ['.max'],
                      fulfill: {
                        state: {
                          value: "{{ $deps[0] ? \"\u6700\u5927\u503C\u4E3A\" + $deps[0] : \"\" }}"
                        }
                      }
                    }]
                  }
                }
              },
              dateGroup: {
                type: 'void',
                'x-hidden': '{{ !($self.query(".__ruleType__").value() === "date") }}',
                properties: {
                  date: {
                    type: 'string',
                    title: '日期',
                    'x-decorator': 'FormItem',
                    'x-component': 'Select',
                    'x-component-props': {
                      allowClear: true
                    }
                  },
                  message: {
                    type: 'string',
                    title: '提示信息',
                    'x-decorator': 'FormItem',
                    'x-component': 'Input',
                    'x-component-props': {
                      allowClear: true,
                      placeholder: '提示信息'
                    }
                  }
                }
              },
              whitespaceGroup: {
                type: 'void',
                'x-hidden': '{{ !($self.query(".__ruleType__").value() === "whitespace") }}',
                properties: {
                  whitespace: {
                    type: 'string',
                    title: '空格',
                    'x-decorator': 'FormItem',
                    'x-component': 'Switch'
                  },
                  message: {
                    type: 'string',
                    title: '提示信息',
                    'x-component-props': {
                      allowClear: true,
                      placeholder: '提示信息'
                    }
                  }
                }
              },
              precisionGroup: {
                type: 'void',
                'x-hidden': '{{ !($self.query(".__ruleType__").value() === "precision") }}',
                properties: {
                  __num__: {
                    type: 'number',
                    title: '小数精度',
                    'x-decorator': 'FormItem',
                    'x-component': 'InputNumber',
                    'x-component-props': {
                      min: 0,
                      max: 10,
                      precision: 0,
                      step: 1
                    }
                  },
                  pattern: {
                    type: 'string',
                    'x-hidden': true,
                    'x-reactions': [{
                      dependencies: ['.__num__'],
                      fulfill: {
                        state: {
                          value: "{{ $deps[0] > 0 ? new RegExp(\"^-?d+(.d{1,\" + $deps[0] + \"})?$\") : undefined }}"
                        }
                      }
                    }]
                  },
                  message: {
                    type: 'string',
                    title: '提示信息',
                    'x-decorator': 'FormItem',
                    'x-component': 'Input',
                    'x-component-props': {
                      allowClear: true,
                      placeholder: '提示信息'
                    }
                  }
                }
              }
            }
          },
          col3: {
            type: 'void',
            'x-component': 'FormRow.Col',
            'x-component-props': {
              flex: '25px',
              style: {
                display: 'flex',
                alignItems: 'center'
              }
            },
            properties: {
              remove: {
                type: 'void',
                'x-decorator': 'FormItem',
                'x-component': 'ArrayItems.Remove'
              }
            }
          }
        }
      }
    }
  },
  properties: {
    addRuleRules: {
      type: 'void',
      title: '添加验证项',
      'x-component': 'ArrayRuleItems.Addition'
    }
  }
};
export var RuleSetter = observer(function () {
  var schema = useFieldSchema();
  var fieldSchema = useMemo(function () {
    return new Schema(ruleSchema, schema);
  }, []);
  return /*#__PURE__*/React.createElement(SchemaContext.Provider, {
    value: fieldSchema
  }, /*#__PURE__*/React.createElement(ArrayRuleItems, null));
});
var required = {
  key: 'required',
  label: '必填'
};
var pattern = {
  key: 'pattern',
  label: '正则'
};
var __format__ = {
  key: '__format__',
  label: '格式'
};
var len = {
  key: 'len',
  label: '长度'
};
var min = {
  key: 'min',
  label: '最小值'
};
var max = {
  key: 'max',
  label: '最大值'
};
var precision = {
  key: 'precision',
  label: '小数精度'
};
var ruleTypesMap = {
  Input: [required, __format__, pattern, len, min, max],
  TextArea: [required, __format__, pattern, len, min, max],
  InputNumber: [required, min, max, precision]
};
var RuleAddition = observer(function (props) {
  var _array$field$query;
  // 需要限制只能添加一次的规则类型
  var uniqueRules = ['required', '__format__', 'len', 'min', 'max', 'date'];
  var prefixCls = usePrefixCls('ant-formily-array-items');
  var _useStyle = useStyle(prefixCls),
    wrapSSR = _useStyle[0],
    hashId = _useStyle[1];
  var self = useField();
  var rootArray = useContext(SideBaseContext);
  var array = ArrayBase.useArray();
  var value = array.field.value || [];
  var selected = useSelected();
  var definitions = selected.getStatementStateDefinitions();
  var isStateRules = definitions && !isEmpty(definitions);
  var componentTypePath = isStateRules ? '...componentType' : '..componentType';
  var componentTypeField = (_array$field$query = array.field.query(componentTypePath)) === null || _array$field$query === void 0 ? void 0 : _array$field$query.take();
  var componentType = !!rootArray ? componentTypeField === null || componentTypeField === void 0 ? void 0 : componentTypeField.value : FormPath.getIn(self.form.values, FormPath.parse(componentTypePath, array.field.path).toString());
  // 获取已存在的规则类型
  var existingRules = value.map(function (item) {
    return item === null || item === void 0 ? void 0 : item.__ruleType__;
  }).filter(Boolean);
  var ruleTypes = ruleTypesMap[componentType] || [required];
  var items = ruleTypes.filter(function (item) {
    // 过滤已存在的唯一性规则
    return !(uniqueRules.includes(item.key) && existingRules.includes(item.key));
  });
  var onClick = function onClick(_ref) {
    var key = _ref.key,
      domEvent = _ref.domEvent;
    var defaultValue = {
      __ruleType__: key
    };
    if (key === 'required') {
      defaultValue = _extends({}, defaultValue, {
        required: true,
        message: '${name}必填'
      });
    } else if (key === 'pattern') {
      defaultValue = _extends({}, defaultValue, {
        'pattern.type': 'JSExpression'
      });
    }
    if (props.method === 'unshift') {
      var _array$field, _array$field$unshift, _array$props, _array$props$onAdd;
      (_array$field = array.field) === null || _array$field === void 0 ? void 0 : (_array$field$unshift = _array$field.unshift) === null || _array$field$unshift === void 0 ? void 0 : _array$field$unshift.call(_array$field, defaultValue);
      (_array$props = array.props) === null || _array$props === void 0 ? void 0 : (_array$props$onAdd = _array$props.onAdd) === null || _array$props$onAdd === void 0 ? void 0 : _array$props$onAdd.call(_array$props, 0);
    } else {
      var _array$field2, _array$field2$push, _array$props2, _array$props2$onAdd, _array$field3, _array$field3$value;
      (_array$field2 = array.field) === null || _array$field2 === void 0 ? void 0 : (_array$field2$push = _array$field2.push) === null || _array$field2$push === void 0 ? void 0 : _array$field2$push.call(_array$field2, defaultValue);
      (_array$props2 = array.props) === null || _array$props2 === void 0 ? void 0 : (_array$props2$onAdd = _array$props2.onAdd) === null || _array$props2$onAdd === void 0 ? void 0 : _array$props2$onAdd.call(_array$props2, (array === null || array === void 0 ? void 0 : (_array$field3 = array.field) === null || _array$field3 === void 0 ? void 0 : (_array$field3$value = _array$field3.value) === null || _array$field3$value === void 0 ? void 0 : _array$field3$value.length) - 1);
    }
    if (props.onClick) {
      props.onClick(domEvent);
    }
  };
  return wrapSSR(/*#__PURE__*/React.createElement("div", {
    className: cls(prefixCls + "-toobar", hashId, props.className)
  }, /*#__PURE__*/React.createElement(_Dropdown, {
    menu: {
      items: items,
      onClick: onClick
    },
    trigger: ['click']
  }, /*#__PURE__*/React.createElement(_Button, {
    color: "primary",
    variant: "filled",
    disabled: items.length === 0
  }, props.title || self.title)), !!rootArray && /*#__PURE__*/React.createElement(Remove, null)));
});
var InternalArrayRuleItems = observer(function (props) {
  return /*#__PURE__*/React.createElement(ArrayItems, props);
});
export var ArrayRuleItems = Object.assign(InternalArrayRuleItems, {
  Addition: RuleAddition
});
var Remove = /*#__PURE__*/forwardRef(function (props, ref) {
  var _array$field4;
  var index = useIndex(props.index);
  var rootArray = useContext(SideBaseContext);
  var self = useField();
  var array = ArrayBase.useArray();
  var prefixCls = usePrefixCls('ant-formily-array-items');
  var _useStyle2 = useStyle(prefixCls),
    wrapSSR = _useStyle2[0],
    hashId = _useStyle2[1];
  if (!array) return null;
  if (((_array$field4 = array.field) === null || _array$field4 === void 0 ? void 0 : _array$field4.pattern) !== 'editable') return null;
  return wrapSSR(/*#__PURE__*/React.createElement(_Button, _extends({
    type: "ghost"
  }, props, {
    style: _extends({
      padding: 8,
      width: 'auto',
      height: 'auto'
    }, props.style),
    disabled: self === null || self === void 0 ? void 0 : self.disabled,
    className: cls(prefixCls + "-remove", hashId, self !== null && self !== void 0 && self.disabled ? prefixCls + "-remove-disabled" : '', props.className),
    ref: ref,
    onClick: function onClick(e) {
      var _rootArray$props, _rootArray$props$onRe;
      if (self !== null && self !== void 0 && self.disabled) return;
      e.stopPropagation();
      if (props.onClick) {
        props.onClick(e);
        if (e.defaultPrevented) return;
      }
      var path = array.field.path.slice(1).toString();
      if (rootArray) {
        var _rootArray$field;
        var value = (rootArray === null || rootArray === void 0 ? void 0 : (_rootArray$field = rootArray.field) === null || _rootArray$field === void 0 ? void 0 : _rootArray$field.value) || [];
        set(value, path, undefined);
      }
      (_rootArray$props = rootArray.props) === null || _rootArray$props === void 0 ? void 0 : (_rootArray$props$onRe = _rootArray$props.onRemove) === null || _rootArray$props$onRe === void 0 ? void 0 : _rootArray$props$onRe.call(_rootArray$props, index);
    },
    icon: isUndef(props.icon) ? /*#__PURE__*/React.createElement(DeleteOutlined, null) : props.icon
  }), props.title));
});