import _extends from "@babel/runtime/helpers/extends";
import _asyncToGenerator from "@babel/runtime/helpers/asyncToGenerator";
import _createClass from "@babel/runtime/helpers/createClass";
import _regeneratorRuntime from "@babel/runtime/regenerator";
import { obx, computed, makeObservable, action, createModuleEventBus } from '@gant-lowcode/lowcode-editor-core';
import { action as reactiveAction } from '@formily/reactive';
function generateSessionId(nodes) {
  return nodes.map(function (node) {
    return node.id;
  }).sort().join(',');
}
export var SettingsMain = /*#__PURE__*/function () {
  function SettingsMain(editor) {
    this.editor = editor;
    this.emitter = createModuleEventBus('SettingsMain');
    this._sessionId = '';
    this._settings = void 0;
    this.disposeListener = void 0;
    this.designer = void 0;
    this.SchemaField = void 0;
    makeObservable(this, {
      _settings: obx.ref,
      setup: action,
      length: computed,
      componentMeta: computed,
      settings: computed
    });
    this.init();
  }
  var _proto = SettingsMain.prototype;
  _proto.init = /*#__PURE__*/function () {
    var _init = _asyncToGenerator(/*#__PURE__*/_regeneratorRuntime.mark(function _callee() {
      var _this = this;
      var setupSelection, designer;
      return _regeneratorRuntime.wrap(function _callee$(_context) {
        while (1) switch (_context.prev = _context.next) {
          case 0:
            setupSelection = function setupSelection(selection) {
              if (selection) {
                _this.setup(selection.getNodes());
              } else {
                _this.setup([]);
              }
            };
            this.editor.eventBus.on('designer.selection.change', setupSelection);
            this.disposeListener = function () {
              _this.editor.removeListener('designer.selection.change', setupSelection);
            };
            _context.next = 5;
            return this.editor.onceGot('designer');
          case 5:
            designer = _context.sent;
            this.designer = designer;
            setupSelection(designer.currentSelection);
          case 8:
          case "end":
            return _context.stop();
        }
      }, _callee, this);
    }));
    function init() {
      return _init.apply(this, arguments);
    }
    return init;
  }();
  _proto.setup = function setup(nodes) {
    // check nodes change
    var sessionId = generateSessionId(nodes);
    if (sessionId === this._sessionId) {
      return;
    }
    this._sessionId = sessionId;
    if (nodes.length < 1) {
      this._settings = undefined;
      return;
    }
    if (!this.designer) {
      this.designer = nodes[0].document.designer;
    }
    // 当节点只有一个时，复用 node 上挂载的 settingEntry，不会产生平行的两个实例，这样在整个系统中对
    // 某个节点操作的 SettingTopEntry 只有一个实例，后续的 getProp() 也会拿到相同的 SettingField 实例
    if (nodes.length === 1) {
      this._settings = nodes[0].settingEntry;
    } else {
      this._settings = this.designer.createSettingEntry(nodes);
    }
  };
  _proto.skeletonSettingsPaneChange = function skeletonSettingsPaneChange(params) {
    this.editor.eventBus.emit('skeleton.settingsPane.change', params);
  };
  _proto.purge = function purge() {
    this.disposeListener();
    this.emitter.removeAllListeners();
  };
  _proto.useDataSource = function useDataSource(key, options) {
    var _this$editor,
      _this2 = this;
    var setters = (_this$editor = this.editor) === null || _this$editor === void 0 ? void 0 : _this$editor.get('setters');
    var dataSourceService = setters.dataSourceService;
    return function (field) {
      field.loading = true;
      try {
        dataSourceService.resolve(key, field, _extends({}, options, {
          node: _this2.settings.getNode()
        })).then(reactiveAction.bound(function (data) {
          field.dataSource = data;
          field.loading = false;
        }));
      } catch (error) {
        field.setFeedback({
          type: 'error',
          code: 'DATA_FETCH_FAILED',
          messages: [error.message]
        });
        field.loading = false;
      }
    };
  };
  return _createClass(SettingsMain, [{
    key: "length",
    get: function get() {
      var _this$_settings;
      return (_this$_settings = this._settings) === null || _this$_settings === void 0 ? void 0 : _this$_settings.nodes.length;
    }
  }, {
    key: "componentMeta",
    get: function get() {
      var _this$_settings2;
      return (_this$_settings2 = this._settings) === null || _this$_settings2 === void 0 ? void 0 : _this$_settings2.componentMeta;
    }
  }, {
    key: "settings",
    get: function get() {
      return this._settings;
    }
  }, {
    key: "scope",
    get: function get() {
      var _this$editor2;
      var setters = (_this$editor2 = this.editor) === null || _this$editor2 === void 0 ? void 0 : _this$editor2.get('setters');
      var registerScope = setters.registerScope;
      var scopes = registerScope.getScopes();
      return _extends({
        useDataSource: this.useDataSource.bind(this),
        node: this.settings.getNode()
      }, scopes);
    }
  }, {
    key: "components",
    get: function get() {
      var _this$editor3;
      var setters = (_this$editor3 = this.editor) === null || _this$editor3 === void 0 ? void 0 : _this$editor3.get('setters');
      var settersMap = setters === null || setters === void 0 ? void 0 : setters.getSettersMap();
      var customComponents = {};
      settersMap === null || settersMap === void 0 ? void 0 : settersMap.forEach(function (setter) {
        customComponents[setter.type] = setter.component;
      });
      return customComponents;
    }
  }]);
}();